# -*- coding: utf-8 -*-

import os
import json
import Image
from StringIO import StringIO
from datetime import datetime

from django.shortcuts import get_object_or_404, render_to_response
from django.template.loader import render_to_string
from django.template import RequestContext
from django.contrib.auth.decorators import login_required
from django.http import HttpResponse, HttpResponseRedirect
from django.utils.translation import ugettext_lazy as _
from django.core.urlresolvers import reverse_lazy
from django.shortcuts import redirect
from django.contrib.sites.models import Site
from django.contrib import messages
import models as mymodels
import forms as myforms

from sorl.thumbnail import get_thumbnail
from braces.views import LoginRequiredMixin, SuperuserRequiredMixin
from taggit.models import Tag

from django.conf import settings as conf

from django.views.generic import DetailView, ListView, CreateView
from django.views.generic.edit import ProcessFormView

from management.commands.utils import *
from forms import PhotoForm


class TagMixin(object):
    def get_context_data(self, **kwargs):
        context = super(TagMixin, self).get_context_data(**kwargs)
        context['tags'] = Tag.objects.all()
        return context


class Photos(TagMixin, ListView):

    template_name = "rgallery/photos.html"
    context_object_name = "photos"
    paginate_by = 150

    def get_queryset(self):
        return mymodels.Photo.objects.all().filter(status=1, folder__isnull=True).order_by('-capture_date')

    def get_context_data(self, **kwargs):
        context = super(Photos, self).get_context_data(**kwargs)
        folders = mymodels.Folder.objects.filter(status=1)
        form = PhotoForm

        context.update({
            'title': _(u'Photos'),
            'form': form,
            'folders': folders,
        })
        return context


class PhotosFolder(ListView):

    template_name = "rgallery/photos.html"
    context_object_name = "photos"

    def get_queryset(self):
        folder = mymodels.Folder.objects.get(slug=self.kwargs['folder'])
        return mymodels.Photo.objects.all().filter(status=1, folder=folder).order_by('-capture_date')

    def get_context_data(self, **kwargs):
        context = super(PhotosFolder, self).get_context_data(**kwargs)

        context.update({
            'title': _(u'Photos'),
        })
        return context


class PhotosTag(ListView):

    template_name = "rgallery/photos.html"
    model = mymodels.Photo
    context_object_name = "photos"
    paginate_by = 100

    def get_queryset(self):
        return mymodels.Photo.objects.filter(status=1,
            tags__slug=self.kwargs.get('slug', None)).order_by('-capture_date')


class PhotoAddTag(LoginRequiredMixin, SuperuserRequiredMixin, ListView):

    model = mymodels.Photo

    def post(self, request, *args, **kwargs):
        if self.request.is_ajax() and self.request.method == 'POST':
            pk = json.loads(self.request.POST.get('pk', None))
            tags = json.loads(self.request.POST.get('tags', None))
            photo = get_object_or_404(mymodels.Photo, pk=pk)
            photo.tags.clear()
            if len(tags) > 0:
                for tag in tags:
                    try:
                        t = Tag.objects.get(slug=tag)
                        photo.tags.add(t)
                    except:
                        photo.tags.add(tag)
            photo.save()
            rendered = render_to_string('rgallery/_listags.html',
                                        {'record': photo.tags.all()})
        return HttpResponse(rendered);


class PhotoAdd(LoginRequiredMixin, SuperuserRequiredMixin, CreateView):

    model = mymodels.Photo
    form_class = PhotoForm
    success_url = '/'

    def form_valid(self, form):
        self.object = form.save(commit=False)
        # Name (if exists in database)
        try:
            im = mymodels.Photo.objects.get(origen=os.path.basename(form.files['file'].name))
        except:
            nombre_imagen = str(os.path.basename(form.files['file'].name)).replace('/', '').replace(' ','_').replace(':','-')

            # Extracting Exif data
            data_image = img_get_exif(form.files['file'])
            capture_data = datetime.strptime(str(data_image['DateTimeOriginal']), "%Y:%m:%d %H:%M:%S")

            # Name (if exists in disk)
            destdir = os.path.join(conf.PROJECT_DIR, 'media', 'uploads', 'photos')
            if os.path.exists(os.path.join(destdir, nombre_imagen)) == True:
                nombre_imagen = str(os.path.basename(form.files['file'].name)).replace('/', '').replace(' ','_').replace(':','-').replace('.jpg','') + '_1.jpg'
            data = ""
            for c in form.files['file'].chunks():
                data += c
            imagefile  = StringIO(data)
            img = Image.open(imagefile)
            img = img_rotate(img)

            # Save
            img.save(os.path.join(destdir, nombre_imagen), img.format)

            # Add to database
            im = mymodels.Photo(image="uploads/photos/" + nombre_imagen,
                                origen=form.files['file'].name,
                                insert_date=datetime.now(),
                                capture_date=capture_data,
                                status=True)
            im.save()

            # Thumbs
            thumbs = [200, 750, 1000]
            for thumb in thumbs:
                im = get_thumbnail(im, "%sx%s" % (thumb, thumb))

        return HttpResponseRedirect(self.get_success_url())


class PhotoDelete(LoginRequiredMixin, SuperuserRequiredMixin, ListView):

    model = mymodels.Photo
    template_name = "rgallery/photo_delete.html"

    def get_context_data(self, **kwargs):
        if self.request.is_ajax() and self.request.method == 'GET':
            pk = json.loads(self.request.GET.get('pk', None))
            photo = get_object_or_404(mymodels.Photo, pk=pk)
            photo.delete()
            return True


class PhotoGetVideo(LoginRequiredMixin, ListView):

    model = mymodels.Photo

    def post(self, request, *args, **kwargs):
        if self.request.is_ajax() and self.request.method == 'POST':
            pk = json.loads(self.request.POST.get('pk', None))
            photo = get_object_or_404(mymodels.Photo, pk=pk)
            rendered = render_to_string('rgallery/_video.html',
                                        {'video': photo.video})
        return HttpResponse(rendered);


class Videos(ListView):

    template_name = "rgallery/videos.html"
    context_object_name = "videos"
    paginate_by = 6

    def get_queryset(self):
        vid = mymodels.Video.objects.all().filter(status=1, folder__isnull=True).order_by('-capture_date')
        print vid
        return vid

    def get_context_data(self, **kwargs):
        context = super(Videos, self).get_context_data(**kwargs)
        context.update({
            'title': _(u'Videos'),
        })
        return context


# -*- coding: utf-8 -*-

from django.conf.urls.defaults import *
from rgallery.views import (Photos,
                            Videos,
                            PhotosFolder,
                            PhotoDelete,
                            PhotoAdd,
                            PhotosTag,
                            PhotoAddTag,
                            PhotoGetVideo)

urlpatterns = patterns('',
    url(r'^$', Photos.as_view(), name='app_gallery-gallery'),
    url(r'^page/(?P<page>\d+)/$', Photos.as_view(), name='app_gallery-gallery-page'),
    url(r'^photo/del/$', PhotoDelete.as_view(), name='app_gallery-photo-del'),
    url(r'^photo/add/$', PhotoAdd.as_view(), name='app_gallery-photo-add'),
    url(r'^photos/tag/(?P<slug>[-\w]+)/$', PhotosTag.as_view(), name='app_gallery-photos-tag'),
    url(r'^photo/add/tag/$', PhotoAddTag.as_view(), name='app_gallery-photo-add-tag'),
    url(r'^photo/get_video/$', PhotoGetVideo.as_view(), name='app_gallery-photo-getvideo'),
    url(r'^videos/$', Videos.as_view(), name='app_gallery-videos'),
    url(r'^videos/page/(?P<page>\d+)/$', Videos.as_view(), name='app_gallery-videos-page'),
    url(r'^(?P<folder>[-_A-Za-z0-9]+)/$', PhotosFolder.as_view(), name='app_gallery-folder'),
)



# -*- coding: utf-8 -*-

def expire_view_cache(
    view_name,
    args=[],
    namespace=None,
    key_prefix=None,
    method="GET"):

    """
    This function allows you to invalidate any view-level cache.

    view_name: view function you wish to invalidate or it's named url pattern
    args: any arguments passed to the view function
    namepace: optioal, if an application namespace is needed
    key prefix: for the @cache_page decorator for the function (if any)

    http://stackoverflow.com/questions/2268417/expire-a-view-cache-in-django
    added: method to request to get the key generating properly
    """

    from django.core.urlresolvers import reverse
    from django.http import HttpRequest
    from django.utils.cache import get_cache_key
    from django.core.cache import cache
    from django.conf import settings
    # create a fake request object
    request = HttpRequest()
    request.method = method
    if settings.USE_I18N:
        request.LANGUAGE_CODE = settings.LANGUAGE_CODE
    # Loookup the request path:
    if namespace:
        view_name = namespace + ":" + view_name
    request.path = reverse(view_name, args=args)
    # get cache key, expire if the cached item exists:
    key = get_cache_key(request, key_prefix=key_prefix)
    if key:
        if cache.get(key):
            cache.set(key, None, 0)
        return True
    return False



# -*- coding: utf-8 -*-

"""
Models for the "rgallery" project
"""

import os
import time
import socket
from django.db import models
from django.contrib.auth.models import User
from django.utils.translation import ugettext_lazy as _
from django.contrib.contenttypes import generic
from django.template.defaultfilters import slugify
from django.conf import settings as conf

from taggit.managers import TaggableManager


class Folder(models.Model):

    """
    Modelo that defines a folder.
    """

    title = models.CharField(_(u'Title'), max_length=255)
    slug = models.SlugField(_(u'Slug'), max_length=255, unique=True)
    password = models.CharField(_(u'Password'), max_length=255, blank=True,
                                null=True, default='')
    status =  models.BooleanField(_(u'Status'), default=True)

    def __unicode__(self):
        return str(self.title)

    class Meta:
        verbose_name = _(u'Folder')
        verbose_name_plural = _(u'Folders')


class Photo(models.Model):

    """
    Modelo that defines a foto.
    """

    title = models.CharField(_(u'Title'), max_length=255, blank=True, null=True)
    image = models.ImageField(_(u'Image'),
                              upload_to='uploads/photos/', blank=True)
    video = models.FileField(_(u'Video'),
                             upload_to='uploads/videos/', blank=True, null=True)
    origen = models.CharField(_(u'Source file'), max_length=255)
    insert_date = models.DateTimeField(_(u'Insert date'))
    capture_date = models.DateTimeField(_(u'Photo date'))
    folder = models.ForeignKey(Folder, related_name='photo', null=True,
                               blank=True, verbose_name=_(u'Folder'))
    tags = TaggableManager()
    status =  models.BooleanField(_(u'Status'), default=True)

    def __unicode__(self):
        return "%s" % str(self.image)

    def split_tags(self):
        t = []
        for tag in self.tags.all():
            t.append(tag.name)
        comma = str(", ".join(t))
        return comma

    def filename(self):
        return os.path.basename(self.image.name)

    def delete(self, *args, **kwargs):
        storage, path = self.image.storage, self.image.path
        vidstorage, vidpath = self.video.storage, self.video.path
        super(Photo, self).delete(*args, **kwargs)
        storage.delete(path)
        storage.delete(vidpath)

    class Meta:
        verbose_name = _(u'Photo')
        verbose_name_plural = _(u'Photos')


class Video(models.Model):

    """
    Modelo that defines a video.
    """

    title = models.CharField(_(u'Title'), max_length=255)
    image = models.ImageField(_(u'Video capture'),
                              upload_to='uploads/photos_videos/', blank=True)
    video = models.FileField(_(u'Video'),
                             upload_to='uploads/videos/', blank=True, null=True)
    origen = models.CharField(_(u'Source file'), max_length=255)
    insert_date = models.DateTimeField(_(u'Insert date'))
    capture_date = models.DateTimeField(_(u'Video date'))
    folder = models.ForeignKey(Folder, related_name='video', null=True,
                               blank=True, verbose_name=_(u'Folder'))
    status =  models.BooleanField(_(u'Status'), default=True)

    def __unicode__(self):
        return str(self.title)

    class Meta:
        verbose_name = _('Video')
        verbose_name_plural = _('Videos')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os, errno
import re
import sys
import time
import datetime
import gzip
import glob

import Image, ExifTags
from ExifTags import TAGS

from optparse import make_option
from django.core.management.base import BaseCommand, CommandError
from django.template import Context, Template
from django.conf import settings as conf
import rgallery.models as mymodels
from datetime import datetime
from pprint import pprint

from sorl.thumbnail import get_thumbnail
from dropbox import client, rest, session


def get_exif(fn):
    """
    data = get_exif('img/2013-04-13 12.17.09.jpg')
    print data
    """
    ret = {}
    print fn
    i = Image.open(fn)
    info = i._getexif()
    try:
        for tag, value in info.items():
            decoded = TAGS.get(tag, tag)
            ret[decoded] = value
    except:
        now = datetime.now()
        ret['DateTimeOriginal'] = now.strftime("%Y:%m:%d %H:%M:%S")

    try:
        str(ret['DateTimeOriginal'])
    except:
        now = datetime.now()
        ret['DateTimeOriginal'] = now.strftime("%Y:%m:%d %H:%M:%S")

    return ret


def mkdir_p(path):
    try:
        os.makedirs(path)
    except OSError as exc: # Python >2.5
        if exc.errno == errno.EEXIST and os.path.isdir(path):
            pass
        else: raise

# Class MUST be named 'Command'
class Command(BaseCommand):

    # Displayed from 'manage.py help mycommand'
    help = """
    Tool that tries to download and parse photos from a Dropbox shared folder,
    saving it on the database and converting the photos to fit the web:

    ./manage.py dropbox_photosync

    1.- First time it connects with Dropbox and stores the token information in
        a token_file
    2.- Next times it read the token from file to connect Dropbox
    3.- Go to the shared folder and check if the image is already on database
    4.- If not, this script download the image, converting it to fit the web and
        save a record in the database.

    To run this script from a crontab task we should do something like this:

    */30 * * * * cd /path/to/rgallery-project/ ; source env/bin/activate ; cd src ; python manage.py dropbox_photosync > /dev/null

    """

    def handle(self, *app_labels, **options):
        """
        The command itself
        """
        # Get your app key and secret from the Dropbox developer website
        APP_KEY = conf.DROPBOX_APP_KEY
        APP_SECRET = conf.DROPBOX_APP_SECRET

        # ACCESS_TYPE should be 'dropbox' or 'app_folder' as configured for your app
        try:
            ACCESS_TYPE = conf.DROPBOX_ACCESS_TYPE
        except:
            ACCESS_TYPE = 'dropbox'

        # Creating dirs
        dropboxdir = os.path.join(conf.PROJECT_DIR, 'media', 'uploads', 'dropbox')
        destdir = os.path.join(conf.PROJECT_DIR, 'media', 'uploads', 'photos')
        if os.path.exists(dropboxdir) == False:
            mkdir_p(dropboxdir)
        if os.path.exists(destdir) == False:
            mkdir_p(destdir)

        # First time
        TOKENS = os.path.join(conf.PROJECT_DIR, 'media', 'uploads', 'dropbox', 'dropbox_token.txt')
        if not os.path.exists(TOKENS):
            sess = session.DropboxSession(APP_KEY, APP_SECRET, ACCESS_TYPE)
            request_token = sess.obtain_request_token()
            url = sess.build_authorize_url(request_token)

            # Make the user sign in and authorize this token
            print "url:", url
            print "Please visit this website and press the 'Allow' button, then hit 'Enter' here."
            raw_input()

            # This will fail if the user didn't visit the above URL
            access_token = sess.obtain_access_token(request_token)

            token_file = open(TOKENS,'w')
            token_file.write("%s|%s" % (access_token.key,access_token.secret) )
            token_file.close()
        else:
            token_file = open(TOKENS)
            token_key,token_secret = token_file.read().split('|')
            token_file.close()
            sess = session.DropboxSession(APP_KEY,APP_SECRET, ACCESS_TYPE )
            sess.set_token(token_key,token_secret)

        c = client.DropboxClient(sess)

        #print "linked account:", c.account_info()

        repetidas = 0
        descargadas = 0
        total = 0
        size = 1000, 1000

        # DROPBOX_PATH: If the access is 'dropbox' you have to specify the folder
        # If it's 'app_folder' it's ok with /
        try:
            DROPBOX_PATH = conf.DROPBOX_ACCESS_PATH
        except:
            DROPBOX_PATH = '/'

        folder_metadata = c.metadata(DROPBOX_PATH)
        #print "*"*80
        #print "metadata:", folder_metadata
        #print "*"*80
        #print "contents:", folder_metadata['contents']

        print "-"*80
        for cont in folder_metadata['contents']:
            print cont

        for cont in folder_metadata['contents']:
            try:
                str(cont['mime_type'])
            except:
                cont['mime_type'] = ''

            if cont['mime_type'] == 'image/jpeg':
                origen_name = os.path.basename(cont['path'])
                #print "origen_name=%s" % origen_name
                try:
                    im = mymodels.Photo.objects.get(origen=os.path.basename(cont['path']))
                    repetidas += 1
                    print "[***] Repetida: %s" % os.path.basename(cont['path'])
                except:
                    print "[***] La imagen %s No esta en bd, descargar y agregar a bbdd y hacer thumb" % os.path.basename(cont['path'])
                    nombre_imagen = str(cont['path']).replace('/', '').replace(' ','_').replace(':','-')

                    # Descargar
                    f, metadata = c.get_file_and_metadata(cont['path'])
                    out = open(dropboxdir + '/' + nombre_imagen, 'wb')
                    out.write(f.read())
                    out.close()
                    print "      Descargada"

                    # Meter
                    data_image = get_exif(dropboxdir + '/' + nombre_imagen)
                    capture_data = datetime.strptime(str(data_image['DateTimeOriginal']), "%Y:%m:%d %H:%M:%S")
                    im = mymodels.Photo(image="uploads/photos/" + nombre_imagen, origen=os.path.basename(cont['path']), insert_date=datetime.now(), capture_date=capture_data, status=True)
                    im.save()
                    print "      Agregada a bbdd"
                    im2 = Image.open(dropboxdir + '/' + nombre_imagen)

                    try:
                        # Rotar si es necesario
                        for orientation in ExifTags.TAGS.keys() :
                            if ExifTags.TAGS[orientation]=='Orientation' : break
                        exif=dict(im2._getexif().items())
                        if exif[orientation] == 3 :
                            im2=im2.rotate(180, expand=True)
                        elif exif[orientation] == 6 :
                            im2=im2.rotate(270, expand=True)
                        elif exif[orientation] == 8 :
                            im2=im2.rotate(90, expand=True)
                    except:
                        pass

                    im2.thumbnail(size, Image.ANTIALIAS)
                    if os.path.exists(destdir + '/' + nombre_imagen) == False:
                        im2.save(destdir + '/' + nombre_imagen)
                        print "      Thumbnail en: %s/%s" % (destdir, nombre_imagen)
                    else:
                        descargadas += 1
                        nombre_imagen_repe = str(cont['path']).replace('/','').replace(' ','_').replace(':','-').replace('.jpg','') + '_1.jpg'
                        im.image = "uploads/photos/" + nombre_imagen_repe
                        im.save()
                        im2.save(destdir + '/' + nombre_imagen_repe)
                        print "      Duplicada en disco, cambiado nombre a: %s" % nombre_imagen_repe

                    print "Haciendo thumb 200x200"
                    im = get_thumbnail(im, "200x200")
                    if im.width > 750 and im.height > 750:
                        print "Haciendo thumb 750x750"
                        im = get_thumbnail(im, "750x750")
                total += 1
            #print "-"*80

        #print "*"*80

        print "[*** Resumen ***]"
        print "[***] Repetidas en db: %s" % repetidas
        print "[***] Repetidas en disco: %s" % descargadas
        print "[***] Total: %s" % total

        print "[***] Haciendo thumbs (200x200 y 750x750) de las que no tienen todavía"
        conf.THUMBNAIL_DEBUG = True
        allphotos = mymodels.Photo.objects.all()
        for i,p in enumerate(allphotos):
            print "%s - %s" % (i, p)
            c = Context({'image': p})
            t = Template('{% load thumbnail %}{% thumbnail image "200x200" crop="top" as img %}{{ img.url }}{% endthumbnail %}')
            t.render(c)
            im = get_thumbnail(p, "200x200")
            if p.image.width > 750 and p.image.height > 750:
                im = get_thumbnail(p, "750x750")
        print "[***] Hecho."




#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os, errno
import re
import sys
import time
import datetime
import gzip
import glob

from hachoir_core.error import HachoirError
from hachoir_core.cmd_line import unicodeFilename
from hachoir_parser import createParser
from hachoir_core.tools import makePrintable
from hachoir_metadata import extractMetadata
from hachoir_core.i18n import getTerminalCharset

from shutil import copyfile
from optparse import make_option
from django.core.management.base import BaseCommand, CommandError
from django.conf import settings as conf
import rgallery.models as mymodels
from datetime import datetime
from pprint import pprint

from dropbox import client, rest, session


def get_exif(fn):
    """
    data = get_exif('img/2013-04-13 12.17.09.jpg')
    print data
    """

    filename = fn
    filename, realname = unicodeFilename(filename), filename
    parser = createParser(filename, realname)
    if not parser:
        print "[EEE] No puedo parsear %s" % filename
        exit(1)
    try:
        metadata = extractMetadata(parser)
    except HachoirError, err:
        print "[EEE] Metadata extraction error: %s" % unicode(err)
        metadata = None
    if not metadata:
        print "[EEE] Unable to extract metadata"
        exit(1)

    text = metadata.exportPlaintext()
    charset = getTerminalCharset()
    for line in text:
        mes = makePrintable(line, charset)
        if 'Creation date' in mes:
            return mes.split(": ")[1]

    #ret = {}
    #i = Image.open(fn)
    #info = i._getexif()
    #for tag, value in info.items():
    #    decoded = TAGS.get(tag, tag)
    #    ret[decoded] = value
    #return ret


def mkdir_p(path):
    try:
        os.makedirs(path)
    except OSError as exc: # Python >2.5
        if exc.errno == errno.EEXIST and os.path.isdir(path):
            pass
        else: raise

# Class MUST be named 'Command'
class Command(BaseCommand):

    # Displayed from 'manage.py help mycommand'
    help = """
    Tool that tries to download and parse photos from a Dropbox shared folder,
    saving it on the database and converting the photos to fit the web:

    ./manage.py dropbox_videosync

    1.- First time it connects with Dropbox and stores the token information in
        a token_file
    2.- Next times it read the token from file to connect Dropbox
    3.- Go to the shared folder and check if the image is already on database
    4.- If not, this script download the image, converting it to fit the web and
        save a record in the database.

    To run this script from a crontab task we should do something like this:

    */30 * * * * cd /path/to/rgallery-project/ ; source env/bin/activate ; cd src ; python manage.py dropbox_videosync > /dev/null

    """

    def handle(self, *app_labels, **options):
        """
        The command itself
        """
        # Get your app key and secret from the Dropbox developer website
        APP_KEY = conf.DROPBOX_APP_KEY
        APP_SECRET = conf.DROPBOX_APP_SECRET

        # ACCESS_TYPE should be 'dropbox' or 'app_folder' as configured for your app
        try:
            ACCESS_TYPE = conf.DROPBOX_ACCESS_TYPE
        except:
            ACCESS_TYPE = 'dropbox'

        # Creating dirs
        dropboxdir = os.path.join(conf.PROJECT_DIR, 'media', 'uploads', 'dropbox')
        destdir = os.path.join(conf.PROJECT_DIR, 'media', 'uploads', 'videos')
        photosvideos = os.path.join(conf.PROJECT_DIR, 'media', 'uploads', 'photos_videos')
        if os.path.exists(dropboxdir) == False:
            mkdir_p(dropboxdir)
        if os.path.exists(destdir) == False:
            mkdir_p(destdir)
        if os.path.exists(photosvideos) == False:
            mkdir_p(photosvideos)

        # First time
        TOKENS = os.path.join(conf.PROJECT_DIR, 'media', 'uploads', 'dropbox', 'dropbox_token.txt')
        if not os.path.exists(TOKENS):
            sess = session.DropboxSession(APP_KEY, APP_SECRET, ACCESS_TYPE)
            request_token = sess.obtain_request_token()
            url = sess.build_authorize_url(request_token)

            # Make the user sign in and authorize this token
            print "url:", url
            print "Please visit this website and press the 'Allow' button, then hit 'Enter' here."
            raw_input()

            # This will fail if the user didn't visit the above URL
            access_token = sess.obtain_access_token(request_token)

            token_file = open(TOKENS,'w')
            token_file.write("%s|%s" % (access_token.key,access_token.secret) )
            token_file.close()
        else:
            token_file = open(TOKENS)
            token_key,token_secret = token_file.read().split('|')
            token_file.close()
            sess = session.DropboxSession(APP_KEY,APP_SECRET, ACCESS_TYPE )
            sess.set_token(token_key,token_secret)

        c = client.DropboxClient(sess)

        #print "linked account:", c.account_info()

        repetidas = 0
        descargadas = 0
        total = 0
        size = 1000, 1000

        folder_metadata = c.metadata('/')
        #print "*"*80
        #print "metadata:", folder_metadata
        #print "*"*80
        #print "contents:", folder_metadata['contents']

        print "-"*80
        for cont in folder_metadata['contents']:
            try:
                str(cont['mime_type'])
            except:
                cont['mime_type'] = ''

            if cont['mime_type'] == 'video/mp4' or cont['mime_type'] == 'video/3gpp':
                origen_name = os.path.basename(cont['path'])
                print "origen_name=%s" % origen_name
                try:
                    im = mymodels.Video.objects.get(origen=os.path.basename(cont['path']))
                    repetidas += 1
                    print "[***] Repetido: %s" % os.path.basename(cont['path'])
                except:
                    print "[***] El video %s No esta en bd, descargar y agregar a bbdd y hacer thumb" % os.path.basename(cont['path'])
                    nombre_imagen = str(cont['path']).replace('/', '').replace(' ','_').replace(':','-')

                    # Descargar
                    f, metadata = c.get_file_and_metadata(cont['path'])
                    out = open(dropboxdir + '/' + nombre_imagen, 'wb')
                    out.write(f.read())
                    out.close()
                    print "      [i] Descargado"

                    # Meter
                    data_image = get_exif(dropboxdir + '/' + nombre_imagen)
                    print "      [i] Nombre imagen: %s" % nombre_imagen
                    print "      [i] Exif data: %s" % data_image

                    # Convertir con FFMPEG      (en principio no hace falta convertir...)
                    # Hacer thumb               (...pero hace falta hacer thumb)

                    # ffmpeg -i [sourcefile.avi] -acodec mp3 -ar 22050 -ab 32 -f flv -s 320×240 [destfile.flv]
                    # flvtool2 -U [flvfile]
                    # ffmpeg -y -i [videofile] -vframes 1 -ss 00:00:02 -an -vcodec png -f rawvideo -s 320×240 [thumbnailimage.png]

                    #flvfile = conf.MEDIA_ROOT + 'video/flv/' + filename_wo_ext + '.flv'
                    #flvfile_url = conf.MEDIA_URL + 'video/flv/' + filename_wo_ext + '.flv'

                    video_filepath = conf.MEDIA_ROOT + '/uploads/dropbox/' + nombre_imagen
                    video_filepath_final = conf.MEDIA_ROOT + '/uploads/videos/' + nombre_imagen
                    video_filepath_final_url = conf.MEDIA_URL + 'uploads/videos/' + nombre_imagen

                    video_thumbfile = conf.MEDIA_ROOT + '/uploads/photos_videos/' + nombre_imagen + '.png'
                    video_thumbfile_url = 'uploads/photos_videos/' + nombre_imagen + '.png'

                    print "      [i] thumbfile: %s" % video_thumbfile
                    print "      [i] thumburl: %s" % video_thumbfile_url

                    """
                    #convertvideo = "%s -i %s -acodec %s -ar 22050 -ab 32 -f flv -s %s %s" % (conf.ZERO14_FFMPEG, video_filepath,
                    #                                                 conf.ZERO14_FFMPEG_ACODEC, conf.ZERO14_FFMPEG_VIDEO_SIZE,
                    #                                                 flvfile)
                    # MarcosBL:
                    # ffmpeg -vstats_file /tmp/vstats -i "$1" -f mp4 -vcodec libx264 -r 25 -b 560000 -ab 64 -ar 44100 -threads 0 "$2" 2>/dev/null &
                    """
                    ZERO14_FFMPEG='/opt/local/bin/ffmpeg'
                    ZERO14_FFMPEG_VCODEC_THUMB='png'
                    ZERO14_FFMPEG_THUMB_SIZE='320x240'
                    grabimage = "%s -y -i %s -vframes 1 -ss 00:00:02 -an -vcodec %s -f rawvideo -s %s %s " % (ZERO14_FFMPEG, video_filepath,
                                                                      ZERO14_FFMPEG_VCODEC_THUMB,
                                                                      ZERO14_FFMPEG_THUMB_SIZE,
                                                                      video_thumbfile)
                    #os.system(convertvideo)
                    os.system(grabimage)

                    # Copiar video a ubicación correspondiente
                    copyfile(video_filepath, video_filepath_final)

                    # Agregar a base de datos
                    capture_data = datetime.strptime(str(data_image), "%Y-%m-%d %H:%M:%S")
                    vid = mymodels.Video(title=capture_data, image=video_thumbfile_url, video=video_filepath_final_url, origen=os.path.basename(cont['path']), insert_date=datetime.now(), capture_date=capture_data, status=True)
                    vid.save()
                    print "      Agregado a bbdd"

                total += 1
            #print "-"*80

        #print "*"*80

        print "[*** Resumen ***]"
        print "[***] Repetidos en db: %s" % repetidas
        print "[***] Repetidos en disco: %s" % descargadas
        print "[***] Total: %s" % total



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os, errno
import re
import sys
import time
import datetime
import gzip
import glob
from datetime import datetime
from pprint import pprint

import Image, ExifTags
from ExifTags import TAGS

from optparse import make_option
from django.core.management.base import BaseCommand, CommandError
from django.template.defaultfilters import slugify
from django.template import Context, Template
from django.conf import settings as conf
from sorl.thumbnail import get_thumbnail

import rgallery.models as mymodels
from utils import *


# Class MUST be named 'Command'
class Command(BaseCommand):

    # Displayed from 'manage.py help mycommand'
    help = """
    Tool that tries to download and parse photos from a source bucket,
    saving it on the database and converting the photos to fit the web:

    ./manage.py mediasync

    1.- First time it connects with the bucket and if it's Dropbox, stores the
        token information in a token_file
    2.- Next times it read the token from file to connect (only Dropbox)
    3.- Go to the shared folder and check if the image is already on database
    4.- If not, this script download the image, converting it to fit the web and
        save a record in the database.

    To run this script from a crontab task we should do something like this:

    */30 * * * * cd /path/to/rgallery-project/ ; source env/bin/activate ; cd src ; python manage.py mediasync > /dev/null

    Options:
        --storage=[dropbox|file]
        --source=/path/to/photos (only when storage=file)

    """

    option_list = BaseCommand.option_list + (
            make_option(
                '--storage',
                dest='storage',
                default='dropbox',
                help='Backend storage where the photos are'
            ),
            make_option(
                '--source',
                dest='source',
                default='/Users/oscar/Desktop/Peques/',
                help='Where the photos are in --storage=file'
            ),
        )


    def handle(self, *app_labels, **options):
        """
        The command itself
        """

        # Vars and folders
        storage = options['storage']
        source = options['source']
        img_duped = 0
        vid_duped = 0
        img_total = 0
        vid_total = 0
        total = 0
        thumbs = [60, 200, 750, 1000]

        # Handle connection with the backend (dinamically importing backend)
        # backend_file, backend_dropbox...
        backend = __import__('rgallery.management.commands.backend_%s' % storage, fromlist='*')

        # Set dirs (src, dest)
        srcdir, photodestdir, videodestdir = backend.set_dirs(source)

        # Handle connection with the backend
        client, bucket = backend.get_contents(srcdir)

        print "-"*80
        print "[***] Importing..."
        print ""

        for file in bucket:
            filepath = backend.filepath(file)
            if backend.is_image(file):
                try:
                    im = mymodels.Photo.objects.get(origen=os.path.basename(filepath))
                    img_duped += 1
                    print "%04d - Duped: %s" % (img_duped, os.path.basename(filepath))
                except:
                    # Downloading
                    print "%04d - Image %s not in database, downlaoding, thumbing and adding to database" % (total, os.path.basename(filepath))
                    nombre_imagen = namedup(filepath, photodestdir) # If duped adds '_1' to the name
                    img = backend.download(client, file, nombre_imagen, srcdir, photodestdir)
                    print "      Saved: %s/%s" % (photodestdir, nombre_imagen)

                    # Reading EXIF Data
                    data_image = img_get_exif(img)
                    capture_data = datetime.strptime(str(data_image['DateTimeOriginal']), "%Y:%m:%d %H:%M:%S")

                    # Adding to database
                    im = mymodels.Photo(image="uploads/photos/" + nombre_imagen,
                                        origen=os.path.basename(filepath),
                                        insert_date=datetime.now(),
                                        capture_date=capture_data,
                                        status=True)
                    im.save()
                    img_total += 1
                    print "      Added to database (%s)" % im.origen

                    # Open image to modify
                    im2 = Image.open(img) # Open
                    im2 = img_rotate(im2) # Rotate
                    im2.save(os.path.join(photodestdir, nombre_imagen))

                    # Creating thumbs
                    for thumb in thumbs:
                        print "      Thumb %sx%s" % (thumb, thumb)
                        c = Context({'image': im, 'thumb': "%sx%s" % (thumb, thumb)})
                        t = Template('{% load thumbnail %}{% thumbnail image thumb crop="top" as img %}{{ img.url }}{% endthumbnail %}')
                        t.render(c)
                        get_thumbnail(im, "%sx%s" % (thumb, thumb))

            if backend.is_video(file):
                try:
                    im = mymodels.Photo.objects.get(origen=os.path.basename(filepath))
                    vid_duped += 1
                    print "%04d - Duped: %s" % (vid_duped, os.path.basename(filepath))
                except:
                    # Downloading
                    print "%04d - Video %s not in database, downloading, thumbing and adding to database" % (total, os.path.basename(filepath))
                    video_name = namedup(filepath, videodestdir) # If duped adds '_1' to the name
                    video = backend.download(client, file, video_name, srcdir, videodestdir)
                    print "      Saved: %s/%s" % (videodestdir, video_name)

                    # Reading EXIF Data
                    rotate, data_video = video_get_exif(video)
                    capture_data = datetime.strptime(str(data_video), "%Y-%m-%d %H:%M:%S")

                    # Converting + thumb
                    thumbname = video_convert(video, file, srcdir, videodestdir, video_name, rotate)

                    # Adding to database
                    vid = mymodels.Photo(title=capture_data,
                                         image="uploads/videos/" + thumbname,
                                         video="uploads/videos/" + video_name,
                                         origen=os.path.basename(filepath),
                                         insert_date=datetime.now(),
                                         capture_date=capture_data,
                                         status=True)
                    vid.save()
                    vid_total += 1
                    print "      Added to database (%s)" % vid.origen

                    # Creating thumbs of image-video (poster)
                    for thumb in thumbs:
                        print "      Thumb %sx%s" % (thumb, thumb)
                        get_thumbnail(vid.image, "%sx%s" % (thumb, thumb))


            total += 1


        print ""
        print "-"*80
        print ""
        print "[***] Making missing thumbs"
        print ""
        conf.THUMBNAIL_DEBUG = True
        allphotos = mymodels.Photo.objects.all()
        for i,p in enumerate(allphotos):
            print "%04d - %s" % (i, p)
            for thumb in thumbs:
                c = Context({'image': p, 'thumb': "%sx%s" % (thumb, thumb)})
                t = Template('{% load thumbnail %}{% thumbnail image thumb crop="top" as img %}{{ img.url }}{% endthumbnail %}')
                t.render(c)
                get_thumbnail(p, "%sx%s" % (thumb, thumb))

        print ""
        print "-"*80
        print ""
        print "[***] Resume images"
        print ""
        print "[***] Duped images (ddbb): %s" % img_duped
        print "[***] Total images: %s" % img_total
        print ""
        print "[***] Resume videos"
        print ""
        print "[***] Duped videos (ddbb): %s" % vid_duped
        print "[***] Total videos: %s" % vid_total
        print ""
        print "[***] TOTAL: %s" % total
        print ""
        print "-"*80



# -*- coding: utf-8 -*-
import os
import re
import sys
import time
import datetime
import gzip
import glob
from mimetypes import MimeTypes

from django.conf import settings as conf

from utils import *


class File(object):
    path = ""
    mime_type = ""

    # The class "constructor" - It's actually an initializer
    def __init__(self, path):
        self.path = path
        try:
            mime = MimeTypes()
            m = str(mime.guess_type(self.path)[0])
        except:
            m = ''
        self.mime_type = m


def set_dirs(source):
    source = source
    photo_target = os.path.join(conf.PROJECT_DIR, 'media', 'uploads', 'photos')
    video_target = os.path.join(conf.PROJECT_DIR, 'media', 'uploads', 'videos')
    if os.path.exists(photo_target) == False:
        mkdir_p(photo_target)
    if os.path.exists(video_target) == False:
        mkdir_p(video_target)
    return source, photo_target, video_target


def get_contents(srcdir):
    types = (os.path.join(srcdir, '*'),)
    bucket = []
    for files in types:
        bucket.extend(glob.glob(files))

    file2 = []
    for file in bucket:
        file2.append(File(file))

    # Returns False for compatibility reasons
    return False, file2


def filepath(file):
    return file.path


def is_image(file):
    if file.mime_type == 'image/jpeg':
        return True
    return False


def is_video(file):
    if file.mime_type == 'video/mp4' or file.mime_type == 'video/3gpp' or file.mime_type == 'video/quicktime' or file.mime_type == 'video/x-msvideo':
        return True
    return False


def download(client, file, nombre_imagen, srcdir, destdir):
    # Keeps client and destdir arguments for compatibility reasons
    # but they're  not used
    f = open(file.path, 'r')
    img = os.path.join(destdir, nombre_imagen)
    out = open(img, 'wb')
    out.write(f.read())
    out.close()
    return img



# -*- coding: utf-8 -*-
from setuptools import setup, find_packages

setup(
    name='django-rgallery',
    version='0.0.31',
    author=u'Oscar M. Lage Guitian',
    author_email='r0sk10@gmail.com',
    #packages=['rgallery'],
    packages = find_packages(),
    include_package_data = True,
    package_data = {'': ['rgallery/templates', 'rgallery/static','rgallery/fixtures',], 'rgallery-example': ['rgallery-example/*']},
    url='http://bitbucket.org/r0sk/django-rgallery',
    license='BSD licence, see LICENSE file',
    description='Yet another Django Gallery App',
    zip_safe=False,
    long_description=open('README.rst').read(),
    install_requires=[
        "Django < 1.5",
        "South == 0.7.5",
        "PIL == 1.1.7",
        "django-braces == 1.4.0",
        "sorl-thumbnail == 11.12",
        "django-compressor == 1.3",
        "dropbox==1.5.1",
        "django-taggit == 0.12.2",
        #"hachoir-core==1.3.3",
        #"hachoir-metadata==1.3.3",
        #"hachoir-parser==1.3.4",
        # ffprobe
    ],
    keywords = "django application gallery",
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)



# -*- coding: utf-8 -*-

import os, errno
import re
import sys
import time
import datetime
import gzip
import glob
from datetime import datetime
from pprint import pprint
from subprocess import Popen, PIPE

from django.core.management.base import BaseCommand, CommandError
from django.template.defaultfilters import slugify
from django.template import Context, Template
from django.conf import settings as conf

import Image, ExifTags
from ExifTags import TAGS
from sorl.thumbnail import get_thumbnail
from shutil import copyfile

# GENERAL **********************************************************************

def mkdir_p(path):
    try:
        os.makedirs(path)
    except OSError as exc: # Python >2.5
        if exc.errno == errno.EEXIST and os.path.isdir(path):
            pass
        else: raise


def namedup(filepath, photodestdir):
    nombre_imagen = "%s%s" % (slugify(str(os.path.splitext(os.path.basename(filepath))[0]).replace('/', '').replace(' ','_').replace(':','-')),
                            str(os.path.splitext(os.path.basename(filepath))[1]))

    # Descargar original a img (si ya existe _1.jpg)
    if os.path.exists(os.path.join(photodestdir, nombre_imagen)) == True:
        nombre_imagen = "%s_1%s" % (slugify(str(os.path.splitext(os.path.basename(filepath))[0]).replace('/', '').replace(' ','_').replace(':','-')),
                                         str(os.path.splitext(os.path.basename(filepath))[1]))
        print "      Duped in disk, name changed to: %s" % nombre_imagen
    return nombre_imagen


# IMAGES ***********************************************************************

def img_get_exif(fn):
    """
    data = get_exif('img/2013-04-13 12.17.09.jpg')
    print data
    """
    ret = {}
    i = Image.open(fn)
    info = i._getexif()
    try:
        for tag, value in info.items():
            decoded = TAGS.get(tag, tag)
            ret[decoded] = value
    except:
        now = datetime.now()
        ret['DateTimeOriginal'] = now.strftime("%Y:%m:%d %H:%M:%S")

    try:
        str(ret['DateTimeOriginal'])
    except:
        now = datetime.now()
        ret['DateTimeOriginal'] = now.strftime("%Y:%m:%d %H:%M:%S")

    return ret


def img_rotate(im2):
    try:
        # Rotar si es necesario
        for orientation in ExifTags.TAGS.keys():
            if ExifTags.TAGS[orientation]=='Orientation':
                break
        exif=dict(im2._getexif().items())
        if exif[orientation] == 3:
            im2=im2.rotate(180, expand=True)
        elif exif[orientation] == 6:
            im2=im2.rotate(270, expand=True)
        elif exif[orientation] == 8:
            im2=im2.rotate(90, expand=True)
    except:
        pass

    return im2


# VIDEOS ***********************************************************************

def video_get_exif(fn):
    # data = get_exif('img/2013-04-13 12.17.09.jpg')
    # print data

    rotate = '0'
    creation = datetime.strftime(datetime.now(), "%Y-%m-%d %H:%M:%S")
    process = Popen([conf.FFPROBE, '-show_streams', str(fn)],
                    stdout=PIPE,
                    stderr=PIPE)
    stdout, stderr = process.communicate()

    for line in iter(stdout.splitlines()):
        if re.search(r'TAG:rotate', line):
            rotate = line.split('=')[1]
        if re.search(r'TAG:creation_time', line):
            creation = line.split('=')[1]

    return rotate, creation


def video_convert(video, file, srcdir, destdir, video_name, rotate):
    # Convertir con FFMPEG      (en principio no hace falta convertir...)
    # Hacer thumb               (...pero hace falta hacer thumb)

    # ffmpeg -i [sourcefile.avi] -acodec mp3 -ar 22050 -ab 32 -f flv -s 320×240 [destfile.flv]
    # flvtool2 -U [flvfile]
    # ffmpeg -y -i [videofile] -vframes 1 -ss 00:00:02 -an -vcodec png -f rawvideo -s 320×240 [thumbnailimage.png]
    #
    # convertvideo = "%s -i %s -acodec %s -ar 22050 -ab 32 -f flv -s %s %s" % (conf.ZERO14_FFMPEG, src,
    #                                                 conf.ZERO14_FFMPEG_ACODEC, conf.ZERO14_FFMPEG_VIDEO_SIZE,
    #                                                 flvfile)
    # MarcosBL:
    # ffmpeg -vstats_file /tmp/vstats -i "$1" -f mp4 -vcodec libx264 -r 25 -b 560000 -ab 64 -ar 44100 -threads 0 "$2" 2>/dev/null &

    try:
        src = file.path
    except:
        src = video
    dst = os.path.join(destdir, video_name)
    thumbname = "%s%s" % (slugify(str(os.path.splitext(os.path.basename(video_name))[0])),'.png')
    thumb = os.path.join(destdir, thumbname)

    rotate, data_video = video_get_exif(dst)

    transpose = ''
    size = conf.FFMPEG_THUMB_SIZE
    if rotate == '90':
        transpose = '-vf "transpose=1"'
        size = conf.FFMPEG_THUMB_SIZE_INVERSE

    # Conf
    # FFPROBE='/opt/local/bin/ffprobe'
    # FFMPEG='/opt/local/bin/ffmpeg'
    # FFMPEG_VCODEC_THUMB='png'
    # FFMPEG_THUMB_SIZE='444x250'
    # FFMPEG_THUMB_SIZE_INVERSE='250x444'

    grabimage = "%s -y -i '%s' -vframes 1 -ss 00:00:02 -an -vcodec %s -f rawvideo -s %s %s %s " % (conf.FFMPEG, src,
                                                                                              conf.FFMPEG_VCODEC_THUMB,
                                                                                              size,
                                                                                              transpose,
                                                                                              thumb)
    #os.system(convertvideo)
    #print "      [%s]" % grabimage
    os.system(grabimage)

    # Copy file to dest
    copyfile(src, dst)

    return thumbname